﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using JLHHJSvr.BLL;
using JLHHJSvr.Com;
using JLHHJSvr.DBA.DBModle;
using LJLib.DAL.SQL;
using LJLib.Net.SPI.Server;
using LJLib.Tools.DEncrypt;

namespace JLHHJSvr.Excutor
{
    internal sealed class SaveUserListExcutor : ExcutorBase<SaveUserListRequest, SaveUserListResponse>
    {
        protected override void ExcuteInternal(SaveUserListRequest request, object state, SaveUserListResponse rslt)
        {
            var tokendata = BllHelper.GetToken(request.token);
            if (tokendata == null)
            {
                rslt.ErrMsg = "会话已经中断，请重新登录";
                return;
            }

            if (request.userList == null || request.userList.Count == 0)
            {
                rslt.ErrMsg = "未提交用户信息";
                return;
            }
            foreach (var user in request.userList)
            {
                if (user.userid == null)
                {
                    rslt.ErrMsg = "用户ID异常";
                    return;
                }
                if (string.IsNullOrEmpty(user.usercode))
                {
                    rslt.ErrMsg = "用户名不能为空";
                    return;
                }
                if (string.IsNullOrEmpty(user.username))
                {
                    rslt.ErrMsg = "真实姓名不能为空";
                    return;
                }
                if (string.IsNullOrEmpty(user.sex))
                {
                    rslt.ErrMsg = "性别不能为空";
                    return;
                }
                if (string.IsNullOrEmpty(user.tel))
                {
                    rslt.ErrMsg = "电话不能为空";
                    return;
                }
                if (user.dscrp == null)
                {
                    rslt.ErrMsg = "备注信息异常";
                    return;
                }
                if (user.powerList == null)
                {
                    rslt.ErrMsg = "权限信息列表获取异常";
                    return;
                }
            }

            using (var con = new SqlConnection(GlobalVar.ConnectionString))
            using (var cmd = con.CreateCommand())
            {
                con.Open();

                using (cmd.Transaction = con.BeginTransaction())
                {
                    try
                    {
                        var dtNow = DateTime.Now;
                        foreach (var user in request.userList)
                        {
                            cmd.CommandText = "SELECT userid FROM st_user WHERE usercode = @usercode AND userid <> @userid";
                            cmd.Parameters.Clear();
                            cmd.Parameters.AddWithValue("@usercode", user.usercode);
                            cmd.Parameters.AddWithValue("@userid", user.userid);
                            using (var reader = cmd.ExecuteReader())
                            {
                                if (reader.Read())
                                {
                                    throw new Exception(string.Format("已存在用户名为[{0}]的记录，不能重复添加", user.usercode));
                                }
                            }
                            var fields = "usercode,username,sex,tel,dscrp";
                            if (user.userid == 0)
                            {
                                //新建
                                user.userid = BllHelper.GetID(cmd, "st_user");
                                user.opdate = dtNow;
                                user.opemp = tokendata.username;
                                user.psw = DESEncrypt.Encrypt(user.usercode, "BC493812B6664BECBF44C21C3BB043C4");
                                fields += ",psw,userid,opemp,opdate";
                                DbSqlHelper.Insert(cmd, "st_user", null, user, fields);
                            }
                            else
                            {
                                //修改
                                user.moddate = dtNow;
                                user.modemp = tokendata.username;
                                fields += ",modemp,moddate";
                                DbSqlHelper.Update(cmd, "st_user", null, user, "userid", fields);
                                //删除原有权限信息
                                cmd.CommandText = "DELETE st_user_power WHERE userid = "+user.userid;
                                cmd.Parameters.Clear();
                                cmd.ExecuteNonQuery();
                            }
                            var userPower = new st_user_power {userid = user.userid};
                            foreach (var power in user.powerList)
                            {
                                if (power.inuse == 1)
                                {
                                    userPower.funid = power.funid;
                                    DbSqlHelper.Insert(cmd, userPower, "userid, funid");
                                }
                            }
                        }
                        cmd.Transaction.Commit();
                    }
                    catch (Exception e)
                    {
                        cmd.Transaction.Rollback();
                        rslt.ErrMsg = e.ToString();
                        return;
                    }
                }
            }
        }
    }
}
